	 ______ __  __ _____ _      _____ 
	|  ____|  \/  |_   _| |    |_   _|
	| |__  | \  / | | | | |      | |  
	|  __| | |\/| | | | | |      | |  
	| |____| |  | |_| |_| |____ _| |_ 
	|______|_|  |_|_____|______|_____|

emili
============

```emili``` is an algorithmic framework to instantiate several different 
optimization algorithms to solve hard combinatorial optimization problems. 
The EMILI framework has been designed to be used in automatic algorithm design
systems that use a grammar based method to generate algorithms. To this end, 
the EMILI framework uses a parser to instantiate algorithms at run time.
The framework is focused on the implementation of single solution SLS algorithms
and in particular hybrid stochastic local search algorithms.

This software is [open source](http://opensource.org/) and is distributed
under the terms of the
[BSD 2-Clause License](http://opensource.org/licenses/BSD-2-Clause).

If you use ```emili``` in your scientific work, please cite the works
below; if you do a derivative work, you should say it is derivative and also
cite the papers below:

 *  Federico Pagnozzi and Thomas Stützle. 
    **Automatic design of hybrid stochastic local search algorithms for 
    permutation flowshop problems.** 
    European journal of operational research, 276(2), 409-421, 2019.
    DOI: [10.1016/j.ejor.2019.01.018](https://doi.org/10.1016/j.ejor.2019.01.018)

#### License ####

Copyright (c) 2015, Federico Pagnozzi
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

### Usage ###

EMILI INSTANCE_FILE_PATH PROBLEM < ALGORITHM DESCRIPTION > [-it|-ro time] [rnds seed] [ps]

### How to compile ###
Use cmake for the configuration and then make.

An example: 

*from the source dir

$ mkdir build

$ cd build

$ cmake ../

$ make

###!WARNING This Code is an adhoc modification of the framework  ###

This modification has been made for the paper titled: 
"Automatic design of hybrid stochastic local search algorithms
 for permutation flowshop problems with additional constraints"

If you want to use or contribute to the framework please do not
modify this code and clone the official repository available at:
     
      https://github.com/federicoPagnozzi/emili

### How to run the algorithms used in the paper ### 

The running times used in the paper are based on the number of jobs n
and the number of machines m of the problem instance to solve and are 
calculated as n*m/2*t milliseconds where t is a parameter. In the experiments we
used three values of t 60,120 and 240. The -ro option when used with 
permutation flowshop calculates the running time as n*m*k seconds where k is 
an argument inserted at running time. The conversion is done as follows. 

t value | argument to pass to emili
  60    |      -ro 0.03
 120    |      -ro 0.06
 240    |      -ro 0.12

### Algorithm definitions ###
The algorithm definitions are given in the form PROBLEM <ALGORITHM DESCRIPTION>

### Permutation floshop with sequence dependant set-up times Makespan ###

Algorithm | Configuration to pass to emili
 EMBO     | SDSTPFS_MS embo sth random random true rch random rndmv insert 1 9 5 1 100 100 0.7
 IRstms   | SDSTPFS_MS ils first nrz locmin sdsttainsert true sdstigo 6 pmetro 4.2073 0.0042 0.0441 206
 IGrs     | SDSTPFS_MS ils first neh locmin sdsttainsert true igper 4 rsacc 0.4
 MRSILS   | SDSTPFS_MS ils ris neh true mrsilsp2 5 rndmv transpose 6 igper 8 improve

### Permutation floshop with sequence dependant set-up times total completion time ###

Algorithm | Configuration to pass to emili
 IRsttct  | SDSTPFS_TCT ils ils ils vnd first mneh msnoimprov binsert exchange tinsert msnoimprov igls 5 first random locmin exchange always diversify maxstep 32 igper 9 saacc 2.0027 0.7216 0.0313 308 0.4397 true igper 2 always intensify
 IGrs     | SDSTPFS_TCT ils ris neh true igper 4 rsacc 0.5

### Permutation floshop with sequence dependant set-up times total tardiness ###

Algorithm | Configuration to pass to emili
 IRsttt   | SDSTPFS_T ils ils als mneh first random locmin finsert first random msnoimprov tinsert msnoimprov igper 3 soaacc 3.3614 true rndmv binsert 1 soaacc 0.2631
 IGrs     | SDSTPFS_T ils ris neh true igper 4 rsacc 0.5

### Permutation floshop no-idle makespan ###

Algorithm | Configuration to pass to emili
 IRnims   | NIPFSP_MS ils ils first nrz2 locmin ncstainsert maxstep 150 niig 8 pmetro 3.8648 0.6922 0.014 137 true igls 5 first random locmin ncstainsert pmetro 3.5859 0.0764 0.076 336
 GVNS     | NIPFSP_MS vns lsvnd neh ils niris random locmin niig 8 improve ils niris random locmin rndmv insert 5 improve true pershake rndmv insert 1 rndmv exchange 1 accng improve
 MANEH    | NIPFSP_MS maneh nehrs 1 true best random locmin ncstainsert vns lsvnd neh ils niris random locmin niig 8 improve ils niris random locmin rndmv insert 5 improve locmin  pershake rndmv insert 1 rndmv exchange 1 accng rsacc 0.4  rndmv insert 1 10 0.1 0.8 0.002 0.15 0.3

### Permutation floshop no-idle total completion time  ###

Algorithm | Configuration to pass to emili
 IRnitct  | NIPFSP_TCT vns vnd first nag 10 20 locmin exchange transpose transpose true pershake vrndmv 7 5 finsert transpose vrndmv 1 5 binsert finsert accng karacc 1.0662
 VIGde    | NIPFSP_TCT vigde nag 10 20 nehrs 1 true 30 0.9 0.9

### Permutation floshop no-idle total tardiness  ###

Algorithm | Configuration to pass to emili
 IRnitt   |  NIPFSP_T ils ils first slack locmin exchange maxstep 134 vrndmv 1 9 binsert finsert karacc 3.3172 true tmiigper 10 0 improve
 DTLM     | NIPFSP_T dtlm nehrs 1 random true best random maxstep 0 insert ris random ils lspipe random best random maxstep 0 insert ris random locmin perset rndmv insert 1 rndmv exchange 1 igper 4 improve igper 4 40 0.2 0.7 0.5 0.1

### How to start contributing ###

Check the official repository!

If you discover a bug or want to propose an improvement, feel free
to contact me at federico.pagnozzi@ulb.ac.be

